==> fastapi-users-db-sqlalchemy-4.0.0.tar.gz: fastapi-users-db-sqlalchemy-4.0.0/fastapi_users_db_sqlalchemy/access_token.py <==
import uuid
from datetime import datetime
from typing import Any, Dict, Generic, Optional, Type, TypeVar

from fastapi_users.authentication.strategy.db import AccessTokenDatabase
from fastapi_users.models import ID
from sqlalchemy import Column, ForeignKey, String, select
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.ext.declarative import declared_attr

from fastapi_users_db_sqlalchemy.generics import GUID, TIMESTAMPAware, now_utc


class SQLAlchemyBaseAccessTokenTable(Generic[ID]):
    """Base SQLAlchemy access token table definition."""

    __tablename__ = "accesstoken"

    token: str = Column(String(length=43), primary_key=True)
    created_at: datetime = Column(
        TIMESTAMPAware, index=True, nullable=False, default=now_utc
    )
    user_id: ID


AP_SQLALCHEMY = TypeVar("AP_SQLALCHEMY", bound=SQLAlchemyBaseAccessTokenTable)


class SQLAlchemyBaseAccessTokenTableUUID(SQLAlchemyBaseAccessTokenTable[uuid.UUID]):
    @declared_attr
    def user_id(cls):
        return Column(GUID, ForeignKey("user.id", ondelete="cascade"), nullable=False)


class SQLAlchemyAccessTokenDatabase(
    Generic[AP_SQLALCHEMY], AccessTokenDatabase[AP_SQLALCHEMY]
):
    """
    Access token database adapter for SQLAlchemy.

    :param session: SQLAlchemy session instance.
    :param access_token_table: SQLAlchemy access token model.
    """

    def __init__(
        self,
        session: AsyncSession,
        access_token_table: Type[AP_SQLALCHEMY],
    ):
        self.session = session
        self.access_token_table = access_token_table

    async def get_by_token(
        self, token: str, max_age: Optional[datetime] = None
    ) -> Optional[AP_SQLALCHEMY]:
        statement = select(self.access_token_table).where(
            self.access_token_table.token == token
        )
        if max_age is not None:
            statement = statement.where(self.access_token_table.created_at >= max_age)

        results = await self.session.execute(statement)
        access_token = results.first()
        if access_token is None:
            return None
        return access_token[0]

    async def create(self, create_dict: Dict[str, Any]) -> AP_SQLALCHEMY:
        access_token = self.access_token_table(**create_dict)
        self.session.add(access_token)
        await self.session.commit()
        await self.session.refresh(access_token)
        return access_token

    async def update(
        self, access_token: AP_SQLALCHEMY, update_dict: Dict[str, Any]
    ) -> AP_SQLALCHEMY:
        for key, value in update_dict.items():
            setattr(access_token, key, value)
        self.session.add(access_token)
        await self.session.commit()
        await self.session.refresh(access_token)
        return access_token

    async def delete(self, access_token: AP_SQLALCHEMY) -> None:
        await self.session.delete(access_token)
        await self.session.commit()

==> fastapi-users-db-sqlalchemy-4.0.1.tar.gz: fastapi-users-db-sqlalchemy-4.0.1/fastapi_users_db_sqlalchemy/access_token.py <==
import uuid
from datetime import datetime
from typing import Any, Dict, Generic, Optional, Type

from fastapi_users.authentication.strategy.db import AP, AccessTokenDatabase
from fastapi_users.models import ID
from sqlalchemy import Column, ForeignKey, String, select
from sqlalchemy.ext.asyncio import AsyncSession
from sqlalchemy.ext.declarative import declared_attr

from fastapi_users_db_sqlalchemy.generics import GUID, TIMESTAMPAware, now_utc


class SQLAlchemyBaseAccessTokenTable(Generic[ID]):
    """Base SQLAlchemy access token table definition."""

    __tablename__ = "accesstoken"

    token: str = Column(String(length=43), primary_key=True)
    created_at: datetime = Column(
        TIMESTAMPAware, index=True, nullable=False, default=now_utc
    )
    user_id: ID


class SQLAlchemyBaseAccessTokenTableUUID(SQLAlchemyBaseAccessTokenTable[uuid.UUID]):
    @declared_attr
    def user_id(cls):
        return Column(GUID, ForeignKey("user.id", ondelete="cascade"), nullable=False)


class SQLAlchemyAccessTokenDatabase(Generic[AP], AccessTokenDatabase[AP]):
    """
    Access token database adapter for SQLAlchemy.

    :param session: SQLAlchemy session instance.
    :param access_token_table: SQLAlchemy access token model.
    """

    def __init__(
        self,
        session: AsyncSession,
        access_token_table: Type[AP],
    ):
        self.session = session
        self.access_token_table = access_token_table

    async def get_by_token(
        self, token: str, max_age: Optional[datetime] = None
    ) -> Optional[AP]:
        statement = select(self.access_token_table).where(
            self.access_token_table.token == token
        )
        if max_age is not None:
            statement = statement.where(self.access_token_table.created_at >= max_age)

        results = await self.session.execute(statement)
        access_token = results.first()
        if access_token is None:
            return None
        return access_token[0]

    async def create(self, create_dict: Dict[str, Any]) -> AP:
        access_token = self.access_token_table(**create_dict)
        self.session.add(access_token)
        await self.session.commit()
        await self.session.refresh(access_token)
        return access_token

    async def update(self, access_token: AP, update_dict: Dict[str, Any]) -> AP:
        for key, value in update_dict.items():
            setattr(access_token, key, value)
        self.session.add(access_token)
        await self.session.commit()
        await self.session.refresh(access_token)
        return access_token

    async def delete(self, access_token: AP) -> None:
        await self.session.delete(access_token)
        await self.session.commit()
